#include "Operand.hpp"

#include "String.hpp"

namespace SoftWire
{
	const Specifier Specifier::specifierSet[] =
	{
		{UNKNOWN,	""},

		{NEAR,		"NEAR"},
		{SHORT,		"SHORT"},
	//	{FAR,		"FAR"},

		{BYTE,		"BYTE"},
		{WORD,		"WORD"},
		{DWORD,		"DWORD"},
	//	{TWORD,		"TWORD"},
		{QWORD,		"QWORD"},
		{MMWORD,	"MMWORD"},
		{XMMWORD,	"XMMWORD"},
		{XWORD,		"XWORD"},
		{OWORD,		"OWORD"},

		{PTR,		"PTR"}
	};

	Specifier::Type Specifier::scan(const char *string)
	{
		if(string)
		{
			for(int i = 0; i < sizeof(specifierSet) / sizeof(Specifier); i++)
			{
				if(stricmp(string, specifierSet[i].notation) == 0)
				{
					return specifierSet[i].type;
				}		
			}
		}

		return UNKNOWN;
	}

	bool Operand::isSubtypeOf(Type type, Type baseType)
	{
		return (type & baseType) == type;
	}

	bool Operand::isSubtypeOf(Type baseType) const
	{
		return isSubtypeOf(type, baseType);
	}

	bool Operand::isVoid(Type type)
	{
		return type == VOID;
	}

	bool Operand::isImm(Type type)
	{
		return (type & IMM) == type;
	}

	bool Operand::isReg(Type type)
	{
		return (type & REG) == type;
	}

	bool Operand::isMem(Type type)
	{
		return (type & MEM) == type;
	}

	bool Operand::isR_M(Type type)
	{
		return (type & R_M) == type;
	}

	bool Operand::isStr(Type type)
	{
		return (type & STR) == type;
	}

	bool Operand::isVoid(const Operand &operand)
	{
		return isVoid(operand.type);
	}

	bool Operand::isImm(const Operand &operand)
	{
		return isImm(operand.type);
	}

	bool Operand::isReg(const Operand &operand)
	{
		return isReg(operand.type);
	}

	bool Operand::isMem(const Operand &operand)
	{
		return isMem(operand.type);
	}

	bool Operand::isR_M(const Operand &operand)
	{
		return isR_M(operand.type);
	}

	bool Operand::isStr(const Operand &operand)
	{
		return isStr(operand.type);
	}

	const Operand Operand::registerSet[] =
	{
		{VOID,		""},

		{AL,		"AL", 0},
		{CL,		"CL", 1},
		{REG8,		"DL", 2},
		{REG8,		"BL", 3},
		{REG8,		"AH", 4},
		{REG8,		"CH", 5},
		{REG8,		"DH", 6},
		{REG8,		"BH", 7},

		{AX,		"AX", 0},
		{CX,		"CX", 1},
		{DX,		"DX", 2},
		{REG16,		"BX", 3},
		{REG16,		"SP", 4},
		{REG16,		"BP", 5},
		{REG16,		"SI", 6},
		{REG16,		"DI", 7},

		{EAX,		"EAX", 0},
		{ECX,		"ECX", 1},
		{REG32,		"EDX", 2},
		{REG32,		"EBX", 3},
		{REG32,		"ESP", 4},
		{REG32,		"EBP", 5},
		{REG32,		"ESI", 6},
		{REG32,		"EDI", 7},

		{ST0,		"ST",  0},
		{ST0,		"ST0", 0},
		{FPUREG,	"ST1", 1},
		{FPUREG,	"ST2", 2},
		{FPUREG,	"ST3", 3},
		{FPUREG,	"ST4", 4},
		{FPUREG,	"ST5", 5},
		{FPUREG,	"ST6", 6},
		{FPUREG,	"ST7", 7},
		{ST0,		"ST(0)", 0},
		{FPUREG,	"ST(1)", 1},
		{FPUREG,	"ST(2)", 2},
		{FPUREG,	"ST(3)", 3},
		{FPUREG,	"ST(4)", 4},
		{FPUREG,	"ST(5)", 5},
		{FPUREG,	"ST(6)", 6},
		{FPUREG,	"ST(7)", 7},

		{MMREG,		"MM0", 0},
		{MMREG,		"MM1", 1},
		{MMREG,		"MM2", 2},
		{MMREG,		"MM3", 3},
		{MMREG,		"MM4", 4},
		{MMREG,		"MM5", 5},
		{MMREG,		"MM6", 6},
		{MMREG,		"MM7", 7},

		{XMMREG,	"XMM0", 0},
		{XMMREG,	"XMM1", 1},
		{XMMREG,	"XMM2", 2},
		{XMMREG,	"XMM3", 3},
		{XMMREG,	"XMM4", 4},
		{XMMREG,	"XMM5", 5},
		{XMMREG,	"XMM6", 6},
		{XMMREG,	"XMM7", 7}
	};

	OperandREG Operand::scanReg(const char *string)
	{
		if(string)
		{
			for(int i = 0; i < sizeof(registerSet) / sizeof(Operand); i++)
			{
				if(stricmp(string, registerSet[i].notation) == 0)
				{
					return registerSet[i];
				}
			}
		}

		return NOT_FOUND;
	}

	const Operand Operand::syntaxSet[] =
	{
		{VOID,		""},

		{ONE,		"1"},
		{IMM,		"imm"},
		{IMM8,		"imm8"},
		{IMM16,		"imm16"},
		{IMM32,		"imm32"},

		{AL,		"AL"},
		{AX,		"AX"},
		{EAX,		"EAX"},
		{DX,		"DX"},
		{CL,		"CL"},
		{CX,		"CX"},
		{ECX,		"ECX"},
		{ST0,		"ST0"},

		{REG8,		"reg8"},
		{REG16,		"reg16"},
		{REG32,		"reg32"},
		{FPUREG,	"fpureg"},
		{MMREG,		"mmreg"},
		{XMMREG,	"xmmreg"},

		{MEM,		"mem"},
		{MEM8,		"mem8"},
		{MEM16,		"mem16"},
		{MEM32,		"mem32"},
		{MEM64,		"mem64"},
		{MEM128,	"mem128"},

		{R_M8,		"r/m8"},
		{R_M16,		"r/m16"},
		{R_M32,		"r/m32"},
		{R_M64,		"r/m64"},
		{R_M128,	"r/m128"},

		{XMM32,		"xmm32"},
		{XMM64,		"xmm64"},

		{STR,		"str"}
	};

	Operand::Type Operand::scanSyntax(const char *string)
	{
		if(string)
		{
			for(int i = 0; i < sizeof(registerSet) / sizeof(Operand); i++)
			{
				if(stricmp(string, syntaxSet[i].notation) == 0)
				{
					return syntaxSet[i].type;
				}
			}
		}

		return UNKNOWN;
	}

	const Operand Operand::INIT = {Operand::VOID};
	const Operand Operand::NOT_FOUND = {Operand::UNKNOWN};
}
